"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class StopRyuMsg(google.protobuf.message.Message):
    """Message that the client sends to stop Ryu"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StopRyuMsg = StopRyuMsg

@typing_extensions.final
class StartRyuMsg(google.protobuf.message.Message):
    """Message that the client sends to start Ryu"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PORT_FIELD_NUMBER: builtins.int
    WEB_PORT_FIELD_NUMBER: builtins.int
    CONTROLLER_FIELD_NUMBER: builtins.int
    port: builtins.int
    web_port: builtins.int
    controller: builtins.str
    def __init__(
        self,
        *,
        port: builtins.int = ...,
        web_port: builtins.int = ...,
        controller: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["controller", b"controller", "port", b"port", "web_port", b"web_port"]) -> None: ...

global___StartRyuMsg = StartRyuMsg

@typing_extensions.final
class StopRyuMonitorMsg(google.protobuf.message.Message):
    """Message that the client sends to stop the Ryu monitor"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StopRyuMonitorMsg = StopRyuMonitorMsg

@typing_extensions.final
class StartRyuMonitorMsg(google.protobuf.message.Message):
    """Message that the client sends to start the Ryu monitor"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    KAFKA_IP_FIELD_NUMBER: builtins.int
    KAFKA_PORT_FIELD_NUMBER: builtins.int
    TIME_STEP_LEN_FIELD_NUMBER: builtins.int
    kafka_ip: builtins.str
    kafka_port: builtins.int
    time_step_len: builtins.int
    def __init__(
        self,
        *,
        kafka_ip: builtins.str = ...,
        kafka_port: builtins.int = ...,
        time_step_len: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["kafka_ip", b"kafka_ip", "kafka_port", b"kafka_port", "time_step_len", b"time_step_len"]) -> None: ...

global___StartRyuMonitorMsg = StartRyuMonitorMsg

@typing_extensions.final
class GetRyuStatusMsg(google.protobuf.message.Message):
    """Message that the client sends to extract the Ryu status"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___GetRyuStatusMsg = GetRyuStatusMsg

@typing_extensions.final
class RyuDTO(google.protobuf.message.Message):
    """Message that the server returns when requested by the client, contains info about Ryu"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    RYU_RUNNING_FIELD_NUMBER: builtins.int
    MONITOR_RUNNING_FIELD_NUMBER: builtins.int
    PORT_FIELD_NUMBER: builtins.int
    WEB_PORT_FIELD_NUMBER: builtins.int
    CONTROLLER_FIELD_NUMBER: builtins.int
    KAFKA_IP_FIELD_NUMBER: builtins.int
    KAFKA_PORT_FIELD_NUMBER: builtins.int
    TIME_STEP_LEN_FIELD_NUMBER: builtins.int
    ryu_running: builtins.bool
    monitor_running: builtins.bool
    port: builtins.int
    web_port: builtins.int
    controller: builtins.str
    kafka_ip: builtins.str
    kafka_port: builtins.int
    time_step_len: builtins.int
    def __init__(
        self,
        *,
        ryu_running: builtins.bool = ...,
        monitor_running: builtins.bool = ...,
        port: builtins.int = ...,
        web_port: builtins.int = ...,
        controller: builtins.str = ...,
        kafka_ip: builtins.str = ...,
        kafka_port: builtins.int = ...,
        time_step_len: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["controller", b"controller", "kafka_ip", b"kafka_ip", "kafka_port", b"kafka_port", "monitor_running", b"monitor_running", "port", b"port", "ryu_running", b"ryu_running", "time_step_len", b"time_step_len", "web_port", b"web_port"]) -> None: ...

global___RyuDTO = RyuDTO
