"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class StopSparkMsg(google.protobuf.message.Message):
    """Message that the client sends to stop spark"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StopSparkMsg = StopSparkMsg

@typing_extensions.final
class StartSparkMsg(google.protobuf.message.Message):
    """Message that the client sends to start spark"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StartSparkMsg = StartSparkMsg

@typing_extensions.final
class StopHostMonitorMsg(google.protobuf.message.Message):
    """Message that the client sends to stop the HostMonitor"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StopHostMonitorMsg = StopHostMonitorMsg

@typing_extensions.final
class StartHostMonitorMsg(google.protobuf.message.Message):
    """Message that the client sends to start the HostMonitor"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    KAFKA_IP_FIELD_NUMBER: builtins.int
    KAFKA_PORT_FIELD_NUMBER: builtins.int
    TIME_STEP_LEN_SECONDS_FIELD_NUMBER: builtins.int
    kafka_ip: builtins.str
    kafka_port: builtins.int
    time_step_len_seconds: builtins.int
    def __init__(
        self,
        *,
        kafka_ip: builtins.str = ...,
        kafka_port: builtins.int = ...,
        time_step_len_seconds: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["kafka_ip", b"kafka_ip", "kafka_port", b"kafka_port", "time_step_len_seconds", b"time_step_len_seconds"]) -> None: ...

global___StartHostMonitorMsg = StartHostMonitorMsg

@typing_extensions.final
class StopFilebeatMsg(google.protobuf.message.Message):
    """Message that the client sends to stop filebeat"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StopFilebeatMsg = StopFilebeatMsg

@typing_extensions.final
class StartFilebeatMsg(google.protobuf.message.Message):
    """Message that the client sends to start filebeat"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StartFilebeatMsg = StartFilebeatMsg

@typing_extensions.final
class ConfigFilebeatMsg(google.protobuf.message.Message):
    """Message that the client sends to set the filebeat configuration"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    KIBANA_IP_FIELD_NUMBER: builtins.int
    KIBANA_PORT_FIELD_NUMBER: builtins.int
    ELASTIC_IP_FIELD_NUMBER: builtins.int
    ELASTIC_PORT_FIELD_NUMBER: builtins.int
    NUM_ELASTIC_SHARDS_FIELD_NUMBER: builtins.int
    RELOAD_ENABLED_FIELD_NUMBER: builtins.int
    KAFKA_IP_FIELD_NUMBER: builtins.int
    KAFKA_PORT_FIELD_NUMBER: builtins.int
    KAFKA_FIELD_NUMBER: builtins.int
    KAFKA_TOPICS_FIELD_NUMBER: builtins.int
    FILEBEAT_MODULES_FIELD_NUMBER: builtins.int
    LOG_FILES_PATHS_FIELD_NUMBER: builtins.int
    kibana_ip: builtins.str
    kibana_port: builtins.int
    elastic_ip: builtins.str
    elastic_port: builtins.int
    num_elastic_shards: builtins.int
    reload_enabled: builtins.bool
    kafka_ip: builtins.str
    kafka_port: builtins.int
    kafka: builtins.bool
    @property
    def kafka_topics(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    @property
    def filebeat_modules(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    @property
    def log_files_paths(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    def __init__(
        self,
        *,
        kibana_ip: builtins.str = ...,
        kibana_port: builtins.int = ...,
        elastic_ip: builtins.str = ...,
        elastic_port: builtins.int = ...,
        num_elastic_shards: builtins.int = ...,
        reload_enabled: builtins.bool = ...,
        kafka_ip: builtins.str = ...,
        kafka_port: builtins.int = ...,
        kafka: builtins.bool = ...,
        kafka_topics: collections.abc.Iterable[builtins.str] | None = ...,
        filebeat_modules: collections.abc.Iterable[builtins.str] | None = ...,
        log_files_paths: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["elastic_ip", b"elastic_ip", "elastic_port", b"elastic_port", "filebeat_modules", b"filebeat_modules", "kafka", b"kafka", "kafka_ip", b"kafka_ip", "kafka_port", b"kafka_port", "kafka_topics", b"kafka_topics", "kibana_ip", b"kibana_ip", "kibana_port", b"kibana_port", "log_files_paths", b"log_files_paths", "num_elastic_shards", b"num_elastic_shards", "reload_enabled", b"reload_enabled"]) -> None: ...

global___ConfigFilebeatMsg = ConfigFilebeatMsg

@typing_extensions.final
class StopPacketbeatMsg(google.protobuf.message.Message):
    """Message that the client sends to stop packetbeat"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StopPacketbeatMsg = StopPacketbeatMsg

@typing_extensions.final
class StartPacketbeatMsg(google.protobuf.message.Message):
    """Message that the client sends to start packetbeat"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StartPacketbeatMsg = StartPacketbeatMsg

@typing_extensions.final
class ConfigPacketbeatMsg(google.protobuf.message.Message):
    """Message that the client sends to set the packetbeat configuration"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    KIBANA_IP_FIELD_NUMBER: builtins.int
    KIBANA_PORT_FIELD_NUMBER: builtins.int
    ELASTIC_IP_FIELD_NUMBER: builtins.int
    ELASTIC_PORT_FIELD_NUMBER: builtins.int
    NUM_ELASTIC_SHARDS_FIELD_NUMBER: builtins.int
    kibana_ip: builtins.str
    kibana_port: builtins.int
    elastic_ip: builtins.str
    elastic_port: builtins.int
    num_elastic_shards: builtins.int
    def __init__(
        self,
        *,
        kibana_ip: builtins.str = ...,
        kibana_port: builtins.int = ...,
        elastic_ip: builtins.str = ...,
        elastic_port: builtins.int = ...,
        num_elastic_shards: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["elastic_ip", b"elastic_ip", "elastic_port", b"elastic_port", "kibana_ip", b"kibana_ip", "kibana_port", b"kibana_port", "num_elastic_shards", b"num_elastic_shards"]) -> None: ...

global___ConfigPacketbeatMsg = ConfigPacketbeatMsg

@typing_extensions.final
class StopMetricbeatMsg(google.protobuf.message.Message):
    """Message that the client sends to stop metricbeat"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StopMetricbeatMsg = StopMetricbeatMsg

@typing_extensions.final
class StartMetricbeatMsg(google.protobuf.message.Message):
    """Message that the client sends to start metricbeat"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StartMetricbeatMsg = StartMetricbeatMsg

@typing_extensions.final
class ConfigMetricbeatMsg(google.protobuf.message.Message):
    """Message that the client sends to set the metricbeat configuration"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    KIBANA_IP_FIELD_NUMBER: builtins.int
    KIBANA_PORT_FIELD_NUMBER: builtins.int
    ELASTIC_IP_FIELD_NUMBER: builtins.int
    ELASTIC_PORT_FIELD_NUMBER: builtins.int
    NUM_ELASTIC_SHARDS_FIELD_NUMBER: builtins.int
    KAFKA_IP_FIELD_NUMBER: builtins.int
    KAFKA_PORT_FIELD_NUMBER: builtins.int
    RELOAD_ENABLED_FIELD_NUMBER: builtins.int
    METRICBEAT_MODULES_FIELD_NUMBER: builtins.int
    kibana_ip: builtins.str
    kibana_port: builtins.int
    elastic_ip: builtins.str
    elastic_port: builtins.int
    num_elastic_shards: builtins.int
    kafka_ip: builtins.str
    kafka_port: builtins.int
    reload_enabled: builtins.bool
    @property
    def metricbeat_modules(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    def __init__(
        self,
        *,
        kibana_ip: builtins.str = ...,
        kibana_port: builtins.int = ...,
        elastic_ip: builtins.str = ...,
        elastic_port: builtins.int = ...,
        num_elastic_shards: builtins.int = ...,
        kafka_ip: builtins.str = ...,
        kafka_port: builtins.int = ...,
        reload_enabled: builtins.bool = ...,
        metricbeat_modules: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["elastic_ip", b"elastic_ip", "elastic_port", b"elastic_port", "kafka_ip", b"kafka_ip", "kafka_port", b"kafka_port", "kibana_ip", b"kibana_ip", "kibana_port", b"kibana_port", "metricbeat_modules", b"metricbeat_modules", "num_elastic_shards", b"num_elastic_shards", "reload_enabled", b"reload_enabled"]) -> None: ...

global___ConfigMetricbeatMsg = ConfigMetricbeatMsg

@typing_extensions.final
class GetHostStatusMsg(google.protobuf.message.Message):
    """Message that the client sends to get the status of the HostMonitor"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___GetHostStatusMsg = GetHostStatusMsg

@typing_extensions.final
class HostStatusDTO(google.protobuf.message.Message):
    """Message that the server returns when asked about the status of running processes"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MONITOR_RUNNING_FIELD_NUMBER: builtins.int
    FILEBEAT_RUNNING_FIELD_NUMBER: builtins.int
    PACKETBEAT_RUNNING_FIELD_NUMBER: builtins.int
    METRICBEAT_RUNNING_FIELD_NUMBER: builtins.int
    HEARTBEAT_RUNNING_FIELD_NUMBER: builtins.int
    monitor_running: builtins.bool
    filebeat_running: builtins.bool
    packetbeat_running: builtins.bool
    metricbeat_running: builtins.bool
    heartbeat_running: builtins.bool
    def __init__(
        self,
        *,
        monitor_running: builtins.bool = ...,
        filebeat_running: builtins.bool = ...,
        packetbeat_running: builtins.bool = ...,
        metricbeat_running: builtins.bool = ...,
        heartbeat_running: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["filebeat_running", b"filebeat_running", "heartbeat_running", b"heartbeat_running", "metricbeat_running", b"metricbeat_running", "monitor_running", b"monitor_running", "packetbeat_running", b"packetbeat_running"]) -> None: ...

global___HostStatusDTO = HostStatusDTO

@typing_extensions.final
class GetHostMetricsMsg(google.protobuf.message.Message):
    """Message that the client sends to extract the Host metrics from given timestamps"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FAILED_AUTH_LAST_TS_FIELD_NUMBER: builtins.int
    LOGIN_LAST_TS_FIELD_NUMBER: builtins.int
    failed_auth_last_ts: builtins.float
    login_last_ts: builtins.float
    def __init__(
        self,
        *,
        failed_auth_last_ts: builtins.float = ...,
        login_last_ts: builtins.float = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["failed_auth_last_ts", b"failed_auth_last_ts", "login_last_ts", b"login_last_ts"]) -> None: ...

global___GetHostMetricsMsg = GetHostMetricsMsg

@typing_extensions.final
class HostMetricsDTO(google.protobuf.message.Message):
    """Message that the server returns when asked about the latest host metrics"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NUM_LOGGED_IN_USERS_FIELD_NUMBER: builtins.int
    NUM_FAILED_LOGIN_ATTEMPTS_FIELD_NUMBER: builtins.int
    NUM_OPEN_CONNECTIONS_FIELD_NUMBER: builtins.int
    NUM_LOGIN_EVENTS_FIELD_NUMBER: builtins.int
    NUM_PROCESSES_FIELD_NUMBER: builtins.int
    NUM_USERS_FIELD_NUMBER: builtins.int
    IP_FIELD_NUMBER: builtins.int
    TIMESTAMP_FIELD_NUMBER: builtins.int
    num_logged_in_users: builtins.int
    num_failed_login_attempts: builtins.int
    num_open_connections: builtins.int
    num_login_events: builtins.int
    num_processes: builtins.int
    num_users: builtins.int
    ip: builtins.str
    timestamp: builtins.float
    def __init__(
        self,
        *,
        num_logged_in_users: builtins.int = ...,
        num_failed_login_attempts: builtins.int = ...,
        num_open_connections: builtins.int = ...,
        num_login_events: builtins.int = ...,
        num_processes: builtins.int = ...,
        num_users: builtins.int = ...,
        ip: builtins.str = ...,
        timestamp: builtins.float = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["ip", b"ip", "num_failed_login_attempts", b"num_failed_login_attempts", "num_logged_in_users", b"num_logged_in_users", "num_login_events", b"num_login_events", "num_open_connections", b"num_open_connections", "num_processes", b"num_processes", "num_users", b"num_users", "timestamp", b"timestamp"]) -> None: ...

global___HostMetricsDTO = HostMetricsDTO

@typing_extensions.final
class StopHeartbeatMsg(google.protobuf.message.Message):
    """Message that the client sends to stop heartbeat"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StopHeartbeatMsg = StopHeartbeatMsg

@typing_extensions.final
class StartHeartbeatMsg(google.protobuf.message.Message):
    """Message that the client sends to start heartbeat"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StartHeartbeatMsg = StartHeartbeatMsg

@typing_extensions.final
class ConfigHeartbeatMsg(google.protobuf.message.Message):
    """Message that the client sends to set the heartbeat configuration"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    KIBANA_IP_FIELD_NUMBER: builtins.int
    KIBANA_PORT_FIELD_NUMBER: builtins.int
    ELASTIC_IP_FIELD_NUMBER: builtins.int
    ELASTIC_PORT_FIELD_NUMBER: builtins.int
    NUM_ELASTIC_SHARDS_FIELD_NUMBER: builtins.int
    HOSTS_TO_MONITOR_FIELD_NUMBER: builtins.int
    kibana_ip: builtins.str
    kibana_port: builtins.int
    elastic_ip: builtins.str
    elastic_port: builtins.int
    num_elastic_shards: builtins.int
    @property
    def hosts_to_monitor(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    def __init__(
        self,
        *,
        kibana_ip: builtins.str = ...,
        kibana_port: builtins.int = ...,
        elastic_ip: builtins.str = ...,
        elastic_port: builtins.int = ...,
        num_elastic_shards: builtins.int = ...,
        hosts_to_monitor: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["elastic_ip", b"elastic_ip", "elastic_port", b"elastic_port", "hosts_to_monitor", b"hosts_to_monitor", "kibana_ip", b"kibana_ip", "kibana_port", b"kibana_port", "num_elastic_shards", b"num_elastic_shards"]) -> None: ...

global___ConfigHeartbeatMsg = ConfigHeartbeatMsg
