"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class GetSnortIdsAlertsMsg(google.protobuf.message.Message):
    """Message that the client sends to extract the Snort IDS log statistics from a given timestamp"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TIMESTAMP_FIELD_NUMBER: builtins.int
    LOG_FILE_PATH_FIELD_NUMBER: builtins.int
    timestamp: builtins.float
    log_file_path: builtins.str
    def __init__(
        self,
        *,
        timestamp: builtins.float = ...,
        log_file_path: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["log_file_path", b"log_file_path", "timestamp", b"timestamp"]) -> None: ...

global___GetSnortIdsAlertsMsg = GetSnortIdsAlertsMsg

@typing_extensions.final
class StopSnortIdsMonitorMsg(google.protobuf.message.Message):
    """Message that the client sends to stop the SnortIdsMonitor"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StopSnortIdsMonitorMsg = StopSnortIdsMonitorMsg

@typing_extensions.final
class StartSnortIdsMonitorMsg(google.protobuf.message.Message):
    """Message that the client sends to start the SnortIdsMonitor"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    KAFKA_IP_FIELD_NUMBER: builtins.int
    KAFKA_PORT_FIELD_NUMBER: builtins.int
    LOG_FILE_PATH_FIELD_NUMBER: builtins.int
    TIME_STEP_LEN_SECONDS_FIELD_NUMBER: builtins.int
    kafka_ip: builtins.str
    kafka_port: builtins.int
    log_file_path: builtins.str
    time_step_len_seconds: builtins.int
    def __init__(
        self,
        *,
        kafka_ip: builtins.str = ...,
        kafka_port: builtins.int = ...,
        log_file_path: builtins.str = ...,
        time_step_len_seconds: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["kafka_ip", b"kafka_ip", "kafka_port", b"kafka_port", "log_file_path", b"log_file_path", "time_step_len_seconds", b"time_step_len_seconds"]) -> None: ...

global___StartSnortIdsMonitorMsg = StartSnortIdsMonitorMsg

@typing_extensions.final
class StopSnortIdsMsg(google.protobuf.message.Message):
    """Message that the client sends to stop the Snort IDS"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StopSnortIdsMsg = StopSnortIdsMsg

@typing_extensions.final
class StartSnortIdsMsg(google.protobuf.message.Message):
    """Message that the client sends to start the Snort IDS"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    INGRESS_INTERFACE_FIELD_NUMBER: builtins.int
    EGRESS_INTERFACE_FIELD_NUMBER: builtins.int
    SUBNETMASK_FIELD_NUMBER: builtins.int
    ingress_interface: builtins.str
    egress_interface: builtins.str
    subnetmask: builtins.str
    def __init__(
        self,
        *,
        ingress_interface: builtins.str = ...,
        egress_interface: builtins.str = ...,
        subnetmask: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["egress_interface", b"egress_interface", "ingress_interface", b"ingress_interface", "subnetmask", b"subnetmask"]) -> None: ...

global___StartSnortIdsMsg = StartSnortIdsMsg

@typing_extensions.final
class GetSnortIdsMonitorStatusMsg(google.protobuf.message.Message):
    """Message that the client sends to get the status of the SnortIdsMonitor"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___GetSnortIdsMonitorStatusMsg = GetSnortIdsMonitorStatusMsg

@typing_extensions.final
class SnortIdsMonitorDTO(google.protobuf.message.Message):
    """Message that the server returns when asked about the status of the Snort IDS Monitor thread and Snort itself"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MONITOR_RUNNING_FIELD_NUMBER: builtins.int
    SNORT_IDS_RUNNING_FIELD_NUMBER: builtins.int
    monitor_running: builtins.bool
    snort_ids_running: builtins.bool
    def __init__(
        self,
        *,
        monitor_running: builtins.bool = ...,
        snort_ids_running: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["monitor_running", b"monitor_running", "snort_ids_running", b"snort_ids_running"]) -> None: ...

global___SnortIdsMonitorDTO = SnortIdsMonitorDTO

@typing_extensions.final
class SnortIdsLogDTO(google.protobuf.message.Message):
    """Message that the server returns when requested by the client, contains info about the Snort IDS log"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TIMESTAMP_FIELD_NUMBER: builtins.int
    IP_FIELD_NUMBER: builtins.int
    ATTEMPTED_ADMIN_ALERTS_FIELD_NUMBER: builtins.int
    ATTEMPTED_USER_ALERTS_FIELD_NUMBER: builtins.int
    INAPPROPRIATE_CONTENT_ALERTS_FIELD_NUMBER: builtins.int
    POLICY_VIOLATION_ALERTS_FIELD_NUMBER: builtins.int
    SHELLCODE_DETECT_ALERTS_FIELD_NUMBER: builtins.int
    SUCCESSFUL_ADMIN_ALERTS_FIELD_NUMBER: builtins.int
    SUCCESSFUL_USER_ALERTS_FIELD_NUMBER: builtins.int
    TROJAN_ACTIVITY_ALERTS_FIELD_NUMBER: builtins.int
    UNSUCCESSFUL_USER_ALERTS_FIELD_NUMBER: builtins.int
    WEB_APPLICATION_ATTACK_ALERTS_FIELD_NUMBER: builtins.int
    ATTEMPTED_DOS_ALERTS_FIELD_NUMBER: builtins.int
    ATTEMPTED_RECON_ALERTS_FIELD_NUMBER: builtins.int
    BAD_UNKNOWN_ALERTS_FIELD_NUMBER: builtins.int
    DEFAULT_LOGIN_ATTEMPT_ALERTS_FIELD_NUMBER: builtins.int
    DENIAL_OF_SERVICE_ALERTS_FIELD_NUMBER: builtins.int
    MISC_ATTACK_ALERTS_FIELD_NUMBER: builtins.int
    NON_STANDARD_PROTOCOL_ALERTS_FIELD_NUMBER: builtins.int
    RPC_PORTMAN_DECODE_ALERTS_FIELD_NUMBER: builtins.int
    SUCCESSFUL_DOS_ALERTS_FIELD_NUMBER: builtins.int
    SUCCESSFUL_RECON_LARGESCALE_ALERTS_FIELD_NUMBER: builtins.int
    SUCCESSFUL_RECON_LIMITED_ALERTS_FIELD_NUMBER: builtins.int
    SUSPICIOUS_FILENAME_DETECT_ALERTS_FIELD_NUMBER: builtins.int
    SUSPICIOUS_LOGIN_ALERTS_FIELD_NUMBER: builtins.int
    SYSTEM_CALL_DETECT_ALERTS_FIELD_NUMBER: builtins.int
    UNUSUAL_CLIENT_PORT_CONNECTION_ALERTS_FIELD_NUMBER: builtins.int
    WEB_APPLICATION_ACTIVITY_ALERTS_FIELD_NUMBER: builtins.int
    ICMP_EVENT_ALERTS_FIELD_NUMBER: builtins.int
    MISC_ACTIVITY_ALERTS_FIELD_NUMBER: builtins.int
    NETWORK_SCAN_ALERTS_FIELD_NUMBER: builtins.int
    NOT_SUSPICIOUS_ALERTS_FIELD_NUMBER: builtins.int
    PROTOCOL_COMMAND_DECODE_ALERTS_FIELD_NUMBER: builtins.int
    STRING_DETECT_ALERTS_FIELD_NUMBER: builtins.int
    UNKNOWN_ALERTS_FIELD_NUMBER: builtins.int
    TCP_CONNECTION_ALERTS_FIELD_NUMBER: builtins.int
    PRIORITY_1_ALERTS_FIELD_NUMBER: builtins.int
    PRIORITY_2_ALERTS_FIELD_NUMBER: builtins.int
    PRIORITY_3_ALERTS_FIELD_NUMBER: builtins.int
    PRIORITY_4_ALERTS_FIELD_NUMBER: builtins.int
    TOTAL_ALERTS_FIELD_NUMBER: builtins.int
    WARNING_ALERTS_FIELD_NUMBER: builtins.int
    SEVERE_ALERTS_FIELD_NUMBER: builtins.int
    ALERTS_WEIGHTED_BY_PRIORITY_FIELD_NUMBER: builtins.int
    timestamp: builtins.float
    ip: builtins.str
    attempted_admin_alerts: builtins.int
    attempted_user_alerts: builtins.int
    inappropriate_content_alerts: builtins.int
    policy_violation_alerts: builtins.int
    shellcode_detect_alerts: builtins.int
    successful_admin_alerts: builtins.int
    successful_user_alerts: builtins.int
    trojan_activity_alerts: builtins.int
    unsuccessful_user_alerts: builtins.int
    web_application_attack_alerts: builtins.int
    attempted_dos_alerts: builtins.int
    attempted_recon_alerts: builtins.int
    bad_unknown_alerts: builtins.int
    default_login_attempt_alerts: builtins.int
    denial_of_service_alerts: builtins.int
    misc_attack_alerts: builtins.int
    non_standard_protocol_alerts: builtins.int
    rpc_portman_decode_alerts: builtins.int
    successful_dos_alerts: builtins.int
    successful_recon_largescale_alerts: builtins.int
    successful_recon_limited_alerts: builtins.int
    suspicious_filename_detect_alerts: builtins.int
    suspicious_login_alerts: builtins.int
    system_call_detect_alerts: builtins.int
    unusual_client_port_connection_alerts: builtins.int
    web_application_activity_alerts: builtins.int
    icmp_event_alerts: builtins.int
    misc_activity_alerts: builtins.int
    network_scan_alerts: builtins.int
    not_suspicious_alerts: builtins.int
    protocol_command_decode_alerts: builtins.int
    string_detect_alerts: builtins.int
    unknown_alerts: builtins.int
    tcp_connection_alerts: builtins.int
    priority_1_alerts: builtins.int
    priority_2_alerts: builtins.int
    priority_3_alerts: builtins.int
    priority_4_alerts: builtins.int
    total_alerts: builtins.int
    warning_alerts: builtins.int
    severe_alerts: builtins.int
    alerts_weighted_by_priority: builtins.int
    def __init__(
        self,
        *,
        timestamp: builtins.float = ...,
        ip: builtins.str = ...,
        attempted_admin_alerts: builtins.int = ...,
        attempted_user_alerts: builtins.int = ...,
        inappropriate_content_alerts: builtins.int = ...,
        policy_violation_alerts: builtins.int = ...,
        shellcode_detect_alerts: builtins.int = ...,
        successful_admin_alerts: builtins.int = ...,
        successful_user_alerts: builtins.int = ...,
        trojan_activity_alerts: builtins.int = ...,
        unsuccessful_user_alerts: builtins.int = ...,
        web_application_attack_alerts: builtins.int = ...,
        attempted_dos_alerts: builtins.int = ...,
        attempted_recon_alerts: builtins.int = ...,
        bad_unknown_alerts: builtins.int = ...,
        default_login_attempt_alerts: builtins.int = ...,
        denial_of_service_alerts: builtins.int = ...,
        misc_attack_alerts: builtins.int = ...,
        non_standard_protocol_alerts: builtins.int = ...,
        rpc_portman_decode_alerts: builtins.int = ...,
        successful_dos_alerts: builtins.int = ...,
        successful_recon_largescale_alerts: builtins.int = ...,
        successful_recon_limited_alerts: builtins.int = ...,
        suspicious_filename_detect_alerts: builtins.int = ...,
        suspicious_login_alerts: builtins.int = ...,
        system_call_detect_alerts: builtins.int = ...,
        unusual_client_port_connection_alerts: builtins.int = ...,
        web_application_activity_alerts: builtins.int = ...,
        icmp_event_alerts: builtins.int = ...,
        misc_activity_alerts: builtins.int = ...,
        network_scan_alerts: builtins.int = ...,
        not_suspicious_alerts: builtins.int = ...,
        protocol_command_decode_alerts: builtins.int = ...,
        string_detect_alerts: builtins.int = ...,
        unknown_alerts: builtins.int = ...,
        tcp_connection_alerts: builtins.int = ...,
        priority_1_alerts: builtins.int = ...,
        priority_2_alerts: builtins.int = ...,
        priority_3_alerts: builtins.int = ...,
        priority_4_alerts: builtins.int = ...,
        total_alerts: builtins.int = ...,
        warning_alerts: builtins.int = ...,
        severe_alerts: builtins.int = ...,
        alerts_weighted_by_priority: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["alerts_weighted_by_priority", b"alerts_weighted_by_priority", "attempted_admin_alerts", b"attempted_admin_alerts", "attempted_dos_alerts", b"attempted_dos_alerts", "attempted_recon_alerts", b"attempted_recon_alerts", "attempted_user_alerts", b"attempted_user_alerts", "bad_unknown_alerts", b"bad_unknown_alerts", "default_login_attempt_alerts", b"default_login_attempt_alerts", "denial_of_service_alerts", b"denial_of_service_alerts", "icmp_event_alerts", b"icmp_event_alerts", "inappropriate_content_alerts", b"inappropriate_content_alerts", "ip", b"ip", "misc_activity_alerts", b"misc_activity_alerts", "misc_attack_alerts", b"misc_attack_alerts", "network_scan_alerts", b"network_scan_alerts", "non_standard_protocol_alerts", b"non_standard_protocol_alerts", "not_suspicious_alerts", b"not_suspicious_alerts", "policy_violation_alerts", b"policy_violation_alerts", "priority_1_alerts", b"priority_1_alerts", "priority_2_alerts", b"priority_2_alerts", "priority_3_alerts", b"priority_3_alerts", "priority_4_alerts", b"priority_4_alerts", "protocol_command_decode_alerts", b"protocol_command_decode_alerts", "rpc_portman_decode_alerts", b"rpc_portman_decode_alerts", "severe_alerts", b"severe_alerts", "shellcode_detect_alerts", b"shellcode_detect_alerts", "string_detect_alerts", b"string_detect_alerts", "successful_admin_alerts", b"successful_admin_alerts", "successful_dos_alerts", b"successful_dos_alerts", "successful_recon_largescale_alerts", b"successful_recon_largescale_alerts", "successful_recon_limited_alerts", b"successful_recon_limited_alerts", "successful_user_alerts", b"successful_user_alerts", "suspicious_filename_detect_alerts", b"suspicious_filename_detect_alerts", "suspicious_login_alerts", b"suspicious_login_alerts", "system_call_detect_alerts", b"system_call_detect_alerts", "tcp_connection_alerts", b"tcp_connection_alerts", "timestamp", b"timestamp", "total_alerts", b"total_alerts", "trojan_activity_alerts", b"trojan_activity_alerts", "unknown_alerts", b"unknown_alerts", "unsuccessful_user_alerts", b"unsuccessful_user_alerts", "unusual_client_port_connection_alerts", b"unusual_client_port_connection_alerts", "warning_alerts", b"warning_alerts", "web_application_activity_alerts", b"web_application_activity_alerts", "web_application_attack_alerts", b"web_application_attack_alerts"]) -> None: ...

global___SnortIdsLogDTO = SnortIdsLogDTO
