"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class StopKafkaMsg(google.protobuf.message.Message):
    """Message that the client sends to stop kafka"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StopKafkaMsg = StopKafkaMsg

@typing_extensions.final
class StartKafkaMsg(google.protobuf.message.Message):
    """Message that the client sends to start kafka"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StartKafkaMsg = StartKafkaMsg

@typing_extensions.final
class GetKafkaStatusMsg(google.protobuf.message.Message):
    """Message that the client sends to extract the kafka status"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___GetKafkaStatusMsg = GetKafkaStatusMsg

@typing_extensions.final
class KafkaDTO(google.protobuf.message.Message):
    """Message that the server returns when requested by the client, contains info about Kafka"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    RUNNING_FIELD_NUMBER: builtins.int
    TOPICS_FIELD_NUMBER: builtins.int
    running: builtins.bool
    @property
    def topics(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    def __init__(
        self,
        *,
        running: builtins.bool = ...,
        topics: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["running", b"running", "topics", b"topics"]) -> None: ...

global___KafkaDTO = KafkaDTO

@typing_extensions.final
class CreateTopicMsg(google.protobuf.message.Message):
    """Message that the client sends to create a new Kafka topic"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    PARTITIONS_FIELD_NUMBER: builtins.int
    REPLICAS_FIELD_NUMBER: builtins.int
    RETENTION_TIME_HOURS_FIELD_NUMBER: builtins.int
    name: builtins.str
    partitions: builtins.int
    replicas: builtins.int
    retention_time_hours: builtins.int
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        partitions: builtins.int = ...,
        replicas: builtins.int = ...,
        retention_time_hours: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["name", b"name", "partitions", b"partitions", "replicas", b"replicas", "retention_time_hours", b"retention_time_hours"]) -> None: ...

global___CreateTopicMsg = CreateTopicMsg

@typing_extensions.final
class DeleteTopicMsg(google.protobuf.message.Message):
    """Message that the client sends to delete a Kafka topic"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    name: builtins.str
    def __init__(
        self,
        *,
        name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["name", b"name"]) -> None: ...

global___DeleteTopicMsg = DeleteTopicMsg
