"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class StopTrafficMsg(google.protobuf.message.Message):
    """Message that the client sends to stop the traffic generator"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___StopTrafficMsg = StopTrafficMsg

@typing_extensions.final
class StartTrafficMsg(google.protobuf.message.Message):
    """Message that the client sends to start the traffic generator"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SLEEPTIME_FIELD_NUMBER: builtins.int
    COMMANDS_FIELD_NUMBER: builtins.int
    sleepTime: builtins.int
    @property
    def commands(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    def __init__(
        self,
        *,
        sleepTime: builtins.int = ...,
        commands: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["commands", b"commands", "sleepTime", b"sleepTime"]) -> None: ...

global___StartTrafficMsg = StartTrafficMsg

@typing_extensions.final
class GetTrafficStatusMsg(google.protobuf.message.Message):
    """Message that the client sends to extract the status of the traffic generator"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___GetTrafficStatusMsg = GetTrafficStatusMsg

@typing_extensions.final
class TrafficDTO(google.protobuf.message.Message):
    """Message that the server returns when requested by the client, contains info about the traffic generator"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    RUNNING_FIELD_NUMBER: builtins.int
    SCRIPT_FIELD_NUMBER: builtins.int
    running: builtins.bool
    script: builtins.str
    def __init__(
        self,
        *,
        running: builtins.bool = ...,
        script: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["running", b"running", "script", b"script"]) -> None: ...

global___TrafficDTO = TrafficDTO
